
///////////////////////////////////////////////////////////////////////////////
// The AppVersionCheck class gets the very latest content from Mippin

// Constructor.
function AppVersionCheck() {
	this.verCheckHttpReq=null;
    this.callback = null;
	this.channelOb = null;
}

AppVersionCheck.prototype.abort = function(){
	// stop request
	if (this.verCheckHttpReq.transport)
		this.verCheckHttpReq.transport.abort();
}

// Returns the text of a node.
AppVersionCheck.prototype.getTextOfNode = function(node) {
    var buf = "";
    
    // iterate through all child elements and collect all text to the buffer
    var child = node.firstChild;
    while (child != null) {
        if (child.nodeType == Node.TEXT_NODE || child.nodeType == Node.CDATA_SECTION_NODE) {
            // append text to buffer
            if (buf != "") {
                buf += " ";
            }
            buf += child.nodeValue;
        }
        child = child.nextSibling;
    }
    
    // strip all tags from the buffer
    var strippedBuf = "";
    var textStartPos = -1;
    var tagBalance = 0;
    
    // iterate through the text and append all text to the stripped buffer
    // that is at a tag balance of 0
    for (pos = 0; pos < buf.length; pos++) {
        var c = buf.charAt(pos);
        if (c == '<') {
            // entering a tag
            if (tagBalance == 0 && textStartPos != -1) {
                // everything up to here was valid text
                strippedBuf += buf.substring(textStartPos, pos);
                textStartPos = -1;
            }
            tagBalance++;
        } else if (c == '>') {
            // leaving a tag
            tagBalance--;
            textStartPos = -1;
        } else if (tagBalance == 0 && textStartPos == -1) {
            // first char of text
            textStartPos = pos;
        }
    }
    
    // add remaining text - if any
    if (tagBalance == 0 && textStartPos != -1) {
        strippedBuf += buf.substring(textStartPos, pos);
    }
    
    return strippedBuf;
}

AppVersionCheck.prototype.isOnline = function(adsURL, callback){
	if (window.widget && (typeof window.navigator.onLine == 'number')) {
		switch (window.navigator.onLine) {
			case window.navigator.NetworkAccessAllowed:{
				return true;
			}
			case window.navigator.NetworkAccessible:{
				return true;
			}
			case window.navigator.NetworkNotAllowed:{
				return false;
			}
			default:
			{
				return false;
			}
		}
	} else return true;
}

AppVersionCheck.prototype.versionCheck = function(callback) {
    // remember callback
    this.callback = callback;
    
    // create new XML HTTP request
    this.verCheckHttpReq = new XMLHttpRequest();
    
    // set callback
    var self = this;
    if (!this.isOnline()) {
		this.callback.call(this, {status: "error"});
		return;
	}
   
	this.verCheckHttpReq.onreadystatechange = function() { self.readyStateChangedVersionChk(); };

    // append the current time after the URL to bypass caches
	var fullURL = updateCheckUrl;
    fullURL += "&nocache=" + (new Date().getTime());
    // initiate the request
    try {
		this.verCheckHttpReq.open("GET", fullURL, true);
		this.verCheckHttpReq.send(null);
	} catch (err){
		this.callback.call(this,{ status: "error" });
	}
}

AppVersionCheck.prototype.readyStateChangedVersionChk = function() {
    // complete request?
    if (this.verCheckHttpReq.readyState == 4) {
        // attempt to get response status
        var responseStatus = null;
        try {
            responseStatus = this.verCheckHttpReq.status;
        } catch (noStatusException) {}
        
		// handle the response and call the registered callback
        this.callback.call(this, this.handleResponseVerChk(responseStatus, this.verCheckHttpReq.responseXML));
    }
}

AppVersionCheck.prototype.handleResponseVerChk = function(responseStatus, xmlDoc) {
	try
	{
		if (responseStatus == 200 && xmlDoc != null) {
		   var downloadUrl = null;
		   var downloadVer = null;
		   var updateAvailableTags = xmlDoc.getElementsByTagName("update_available");
		   if(updateAvailableTags!=null && updateAvailableTags.length>0 && this.getTextOfNode(updateAvailableTags[0])=="true")
		   {
		   	    var updateURLs = xmlDoc.getElementsByTagName("update_url");
				if(updateURLs != null && updateURLs.length>0)
				{
					downloadUrl = this.getTextOfNode(updateURLs[0]);
					
					var versions = xmlDoc.getElementsByTagName("update_version");
					if (versions != null && versions.length > 0) {
						downloadVer = this.getTextOfNode(versions[0]);
					}
				}
		   }
	
			if(downloadUrl == null)
			{
				return { status: "error" };
			}else
			{
				return { status: "ok", downloadUrl: downloadUrl, dlVersion: downloadVer };
			}
	    } else {
	        // update failed
	        return { status: "error" };
	    }
    }catch(error){return { status: "error" };}
}
