///////////////////////////////////////////////////////////////////////////////
// The ListView extra for Mippin class implements a vertical list view that hosts controls
// as child components.

// Constructor.
function ListViewMippin(id, caption) {
    if (id != UI_NO_INIT_ID) {
        this.init(id, caption);
    }
}

// ListView inherits from View.
ListViewMippin.prototype = new View(UI_NO_INIT_ID);

// The caption of this view; null if none.
ListViewMippin.prototype.caption = null;

// The caption element of this view.
ListViewMippin.prototype.captionElement = null;

// The caption text element of this view.
ListViewMippin.prototype.captionTextElement = null;

// Root HTML element for controls.
ListViewMippin.prototype.listElement = null;

// List of controls in the view.
ListViewMippin.prototype.controls = null;

// Initializer for ListViewMippin.
ListViewMippin.prototype.init = function(id) {
    uiLogger.debug("ListViewMippin.init(" + id + ")");
    
    // call superclass initializer
    View.prototype.init.call(this, id);
    
    // init control array
    this.controls = [];
	
	// mouse positions
	this.mouseX=0;
	this.mouseY=0;
    
    // set style class name for root element
    this.rootElement.className = "ListViewMippin";
    this.rootElement.id = "ListViewMippinId";

    // create caption and caption text elements
//    this.captionElement = document.createElement("div");
//    this.captionElement.className = "ListViewCaption";
//    this.captionElement.id = "content-topline";
//    this.captionTextElement = document.createElement("div");
//    this.captionTextElement.className = "ListViewCaptionText";
//    this.captionTextElement.id = "ListViewCaptionId";
//    this.captionElement.appendChild(this.captionTextElement);
//    this.rootElement.appendChild(this.captionElement);
    
    // create root element for controls and add to the view root element
    this.listElement = document.createElement("div");
    this.listElement.className = "ListViewControlList";
	this.rootElement.appendChild(this.listElement);
    
    // set the caption
//    this.setCaption(caption);
}

// Returns the caption; null if none.
//ListViewMippin.prototype.getCaption = function() {
//    return this.caption;
//}

// Sets the caption; null if none.
//ListViewMippin.prototype.setCaption = function(caption) {
//    uiLogger.debug("ListViewMippin.setCaption(" + caption + ")");
//    
//    // set the display style
//    this.captionElement.style.display = (caption == null) ? "none" : "block";
//    
//    // set the caption
//    this.caption = caption;
//    this.captionTextElement.innerHTML = (caption == null) ? "" : caption;
//}

// Returns an array of controls in the view.
ListViewMippin.prototype.getControls = function() {
    return this.controls;
}

ListViewMippin.prototype.refreshView = function(isMain,newControls){
	if (this.listElement.childElementCount==0 && this.controls.length>0){
		 this.controls = [];
		 for (var i = 0; i < newControls.length; i++) {
        	var id = "ls";
			if (isMain) id = id+"p";
			id = id+i;
			this.addControl(newControls[i],id);
		}
	}
}

// Adds a control to the view.
ListViewMippin.prototype.addControl = function(control,id) {
    uiLogger.debug("ListViewMippin.addControl(" + control + ")");
    
    // add the control to the controls array and attach it to the list element
	control.setId(control.rootElement,"ls"+id);
    this.controls.push(control);
    this.listElement.appendChild(control.rootElement);
    control.view = this;
}

// Inserts a control to the view before the specified control.
ListViewMippin.prototype.insertControl = function(control, beforeControl) {
    uiLogger.debug("ListViewMippin.insertControl(" + control + ", " + beforeControl + ")");
    
    // iterate through current controls
    for (var i = 0; i < this.controls.length; i++) {
        // is this the control we should insert before?
        if (this.controls[i] == beforeControl) {
            // we found the control to insert before - insert here and connect to list element
            this.controls.splice(i, 0, control);
            this.listElement.insertBefore(control.rootElement, beforeControl.rootElement);
            control.view = this;
            return;
        }
    }
    
    // the control wasn't found so we'll add it last
    this.addControl(control);
}

// Removes a control from the view.
ListViewMippin.prototype.removeControl = function(control) {
    uiLogger.debug("ListViewMippin.removeControl(" + control + ")");
    
    // iterate through current controls
    for (var i = 0; i < this.controls.length; i++) {
        // is this the control we should remove?
        if (this.controls[i] == control) {
            // we found the control to remove - remove it from the list element
            this.controls.splice(i, 1);
            this.listElement.removeChild(control.rootElement);
            control.view = null;
        }
    }
}

ListViewMippin.prototype.removeAllControls = function() {
    uiLogger.debug("ListViewMippin.removeAllControls()");
    
    // iterate through current controls
    for (var i = 0; i < this.controls.length; i++) {
       this.controls.splice(i, 1);
       this.listElement.removeChild(this.controls[i].rootElement);
       this.controls[i].view = null;
    }
}

// Attempts to focus the first focusable control.
ListViewMippin.prototype.focusFirstControl = function() {
    uiLogger.debug("ListViewMippin.focusFirstControl()");
    for (var i = 0; i < this.controls.length; i++) {
        // is this control focusable?
        var control = this.controls[i];
        if (control.isFocusable()) {
            control.setFocused(true);
            break;
        }
    }
}

// Attempts to reset all control focus states.
// Override in subclasses as required.
ListViewMippin.prototype.resetControlFocusStates = function() {
    uiLogger.debug("ListViewMippin.resetControlFocusStates()");
    for (var i = 0; i < this.controls.length; i++) {
        this.controls[i].resetFocusState();
    }
}
